@@ -152,9 +152,8 @@ class ConfigCommands:
             key: The keychain to bind. Examples of valid keychains are `gC`,
                  `<Ctrl-X>` or `<Ctrl-C>a`.
             command: The command to execute, with optional args.
-            mode: A comma-separated list of modes to bind the key in
-                  (default: `normal`). See `:help bindings.commands` for the
-                  available modes.
+            mode: The mode to bind the key in (default: `normal`). See `:help
+                  bindings.commands` for the available modes.
             default: If given, restore a default binding.
         """
         if key is None:
@@ -193,7 +192,7 @@ class ConfigCommands:
         Args:
             key: The keychain to unbind. See the help for `:bind` for the
                   correct syntax for keychains.
-            mode: A mode to unbind the key in (default: `normal`).
+            mode: The mode to unbind the key in (default: `normal`).
                   See `:help bindings.commands` for the available modes.
         """
         with self._handle_config_error():
@@ -275,7 +274,8 @@ class ConfigCommands:
             value: The value to append to the end of the list.
             temp: Add value temporarily until qutebrowser is closed.
         """
-        opt = self._config.get_opt(option)
+        with self._handle_config_error():
+            opt = self._config.get_opt(option)
         valid_list_types = (configtypes.List, configtypes.ListOrValue)
         if not isinstance(opt.typ, valid_list_types):
             raise cmdutils.CommandError(":config-list-add can only be used "
@@ -300,7 +300,8 @@ class ConfigCommands:
             replace: Replace existing values. By default, existing values are
                      not overwritten.
         """
-        opt = self._config.get_opt(option)
+        with self._handle_config_error():
+            opt = self._config.get_opt(option)
         if not isinstance(opt.typ, configtypes.Dict):
             raise cmdutils.CommandError(":config-dict-add can only be used "
                                         "for dicts")
@@ -327,7 +328,8 @@ class ConfigCommands:
             value: The value to remove from the list.
             temp: Remove value temporarily until qutebrowser is closed.
         """
-        opt = self._config.get_opt(option)
+        with self._handle_config_error():
+            opt = self._config.get_opt(option)
         valid_list_types = (configtypes.List, configtypes.ListOrValue)
         if not isinstance(opt.typ, valid_list_types):
             raise cmdutils.CommandError(":config-list-remove can only be used "
@@ -355,7 +357,8 @@ class ConfigCommands:
             key: The key to remove from the dict.
             temp: Remove value temporarily until qutebrowser is closed.
         """
-        opt = self._config.get_opt(option)
+        with self._handle_config_error():
+            opt = self._config.get_opt(option)
         if not isinstance(opt.typ, configtypes.Dict):
             raise cmdutils.CommandError(":config-dict-remove can only be used "
                                         "for dicts")
