@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2017-2018 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2017-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -26,6 +26,7 @@ import html
 import ctypes
 import ctypes.util
 import enum
+import shutil
 
 import attr
 from PyQt5.QtCore import Qt
@@ -33,15 +34,20 @@ from PyQt5.QtWidgets import (QApplication, QDialog, QPushButton, QHBoxLayout,
                              QVBoxLayout, QLabel, QMessageBox)
 from PyQt5.QtNetwork import QSslSocket
 
-from qutebrowser.config import config
-from qutebrowser.utils import usertypes, objreg, version, qtutils, log, utils
+from qutebrowser.config import config, configfiles
+from qutebrowser.utils import (usertypes, objreg, version, qtutils, log, utils,
+                               standarddir)
 from qutebrowser.misc import objects, msgbox
 
 
-_Result = enum.IntEnum(
-    '_Result',
-    ['quit', 'restart', 'restart_webkit', 'restart_webengine'],
-    start=QDialog.Accepted + 1)
+class _Result(enum.IntEnum):
+
+    """The result code returned by the backend problem dialog."""
+
+    quit = QDialog.Accepted + 1
+    restart = QDialog.Accepted + 2
+    restart_webkit = QDialog.Accepted + 3
+    restart_webengine = QDialog.Accepted + 4
 
 
 @attr.s
@@ -174,6 +180,10 @@ def _nvidia_shader_workaround():
     See https://bugs.launchpad.net/ubuntu/+source/python-qt4/+bug/941826
     """
     assert objects.backend == usertypes.Backend.QtWebEngine, objects.backend
+
+    if os.environ.get('QUTE_SKIP_LIBGL_WORKAROUND'):
+        return
+
     libgl = ctypes.util.find_library("GL")
     if libgl is not None:
         ctypes.CDLL(libgl, mode=ctypes.RTLD_GLOBAL)
@@ -234,6 +244,9 @@ def _handle_wayland():
     if has_qt511 and config.val.qt.force_software_rendering == 'chromium':
         return
 
+    if qtutils.version_check('5.11.2', compiled=False):
+        return
+
     buttons = []
     text = "<p>You can work around this in one of the following ways:</p>"
 
@@ -280,7 +293,7 @@ class BackendImports:
 
 def _try_import_backends():
     """Check whether backends can be imported and return BackendImports."""
-    # pylint: disable=unused-variable
+    # pylint: disable=unused-import
     results = BackendImports()
 
     try:
@@ -386,6 +399,29 @@ def _check_backend_modules():
     raise utils.Unreachable
 
 
+def _handle_cache_nuking():
+    """Nuke the QtWebEngine cache if the Qt version changed.
+
+    WORKAROUND for https://bugreports.qt.io/browse/QTBUG-72532
+    """
+    if not configfiles.state.qt_version_changed:
+        return
+
+    # Only nuke the cache in cases where we know there are problems.
+    # It seems these issues started with Qt 5.12.
+    # They should be fixed with Qt 5.12.5:
+    # https://codereview.qt-project.org/c/qt/qtwebengine-chromium/+/265408
+    affected = (qtutils.version_check('5.12', compiled=False) and not
+                qtutils.version_check('5.12.5', compiled=False))
+    if not affected:
+        return
+
+    log.init.info("Qt version changed, nuking QtWebEngine cache")
+    cache_dir = os.path.join(standarddir.cache(), 'webengine')
+    if os.path.exists(cache_dir):
+        shutil.rmtree(cache_dir)
+
+
 def init():
     """Check for various issues related to QtWebKit/QtWebEngine."""
     _check_backend_modules()
@@ -394,6 +430,7 @@ def init():
         _handle_wayland()
         _nvidia_shader_workaround()
         _handle_nouveau_graphics()
+        _handle_cache_nuking()
     else:
         assert objects.backend == usertypes.Backend.QtWebKit, objects.backend
         _handle_ssl_support(fatal=True)
