@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2018 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -24,21 +24,23 @@ Module attributes:
              value.
     MINVALS: A dictionary of C/Qt types (as string) mapped to their minimum
              value.
+    MAX_WORLD_ID: The highest world ID allowed in this version of QtWebEngine.
 """
 
 
 import io
 import operator
 import contextlib
+import typing  # pylint: disable=unused-import,useless-suppression
 
 import pkg_resources
 from PyQt5.QtCore import (qVersion, QEventLoop, QDataStream, QByteArray,
                           QIODevice, QSaveFile, QT_VERSION_STR,
-                          PYQT_VERSION_STR)
+                          PYQT_VERSION_STR, QFileDevice, QObject)
 try:
     from PyQt5.QtWebKit import qWebKitVersion
 except ImportError:  # pragma: no cover
-    qWebKitVersion = None
+    qWebKitVersion = None  # type: ignore  # noqa: N816
 
 
 MAXVALS = {
@@ -60,19 +62,22 @@ class QtOSError(OSError):
         qt_errno: The error attribute of the given QFileDevice, if applicable.
     """
 
-    def __init__(self, dev, msg=None):
+    def __init__(self, dev: QFileDevice, msg: str = None) -> None:
         if msg is None:
             msg = dev.errorString()
 
         super().__init__(msg)
 
+        self.qt_errno = None  # type: typing.Optional[QFileDevice.FileError]
         try:
             self.qt_errno = dev.error()
         except AttributeError:
-            self.qt_errno = None
+            pass
 
 
-def version_check(version, exact=False, compiled=True):
+def version_check(version: str,
+                  exact: bool = False,
+                  compiled: bool = True) -> bool:
     """Check if the Qt runtime version is the version supplied or newer.
 
     Args:
@@ -98,14 +103,18 @@ def version_check(version, exact=False, compiled=True):
     return result
 
 
-def is_new_qtwebkit():
+# WORKAROUND for https://bugreports.qt.io/browse/QTBUG-69904
+MAX_WORLD_ID = 256 if version_check('5.11.2') else 11
+
+
+def is_new_qtwebkit() -> bool:
     """Check if the given version is a new QtWebKit."""
     assert qWebKitVersion is not None
     return (pkg_resources.parse_version(qWebKitVersion()) >
             pkg_resources.parse_version('538.1'))
 
 
-def check_overflow(arg, ctype, fatal=True):
+def check_overflow(arg: int, ctype: str, fatal: bool = True) -> int:
     """Check if the given argument is in bounds for the given type.
 
     Args:
@@ -133,13 +142,13 @@ def check_overflow(arg, ctype, fatal=True):
         return arg
 
 
-def ensure_valid(obj):
+def ensure_valid(obj: QObject) -> None:
     """Ensure a Qt object with an .isValid() method is valid."""
     if not obj.isValid():
         raise QtValueError(obj)
 
 
-def check_qdatastream(stream):
+def check_qdatastream(stream: QDataStream) -> None:
     """Check the status of a QDataStream and raise OSError if it's not ok."""
     status_to_str = {
         QDataStream.Ok: "The data stream is operating normally.",
@@ -153,7 +162,7 @@ def check_qdatastream(stream):
         raise OSError(status_to_str[stream.status()])
 
 
-def serialize(obj):
+def serialize(obj: QObject) -> QByteArray:
     """Serialize an object into a QByteArray."""
     data = QByteArray()
     stream = QDataStream(data, QIODevice.WriteOnly)
@@ -161,20 +170,20 @@ def serialize(obj):
     return data
 
 
-def deserialize(data, obj):
+def deserialize(data: QByteArray, obj: QObject) -> None:
     """Deserialize an object from a QByteArray."""
     stream = QDataStream(data, QIODevice.ReadOnly)
     deserialize_stream(stream, obj)
 
 
-def serialize_stream(stream, obj):
+def serialize_stream(stream: QDataStream, obj: QObject) -> None:
     """Serialize an object into a QDataStream."""
     check_qdatastream(stream)
     stream << obj  # pylint: disable=pointless-statement
     check_qdatastream(stream)
 
 
-def deserialize_stream(stream, obj):
+def deserialize_stream(stream: QDataStream, obj: QObject) -> None:
     """Deserialize a QDataStream into an object."""
     check_qdatastream(stream)
     stream >> obj  # pylint: disable=pointless-statement
@@ -190,11 +199,14 @@ def savefile_open(filename, binary=False, encoding='utf-8'):
         open_ok = f.open(QIODevice.WriteOnly)
         if not open_ok:
             raise QtOSError(f)
+
         if binary:
             new_f = PyQIODevice(f)
         else:
             new_f = io.TextIOWrapper(PyQIODevice(f), encoding=encoding)
+
         yield new_f
+
         new_f.flush()
     except:
         f.cancelWriting()
@@ -214,29 +226,29 @@ class PyQIODevice(io.BufferedIOBase):
         dev: The underlying QIODevice.
     """
 
-    def __init__(self, dev):
+    def __init__(self, dev: QIODevice) -> None:
         super().__init__()
         self.dev = dev
 
-    def __len__(self):
+    def __len__(self) -> int:
         return self.dev.size()
 
-    def _check_open(self):
+    def _check_open(self) -> None:
         """Check if the device is open, raise ValueError if not."""
         if not self.dev.isOpen():
             raise ValueError("IO operation on closed device!")
 
-    def _check_random(self):
+    def _check_random(self) -> None:
         """Check if the device supports random access, raise OSError if not."""
         if not self.seekable():
             raise OSError("Random access not allowed!")
 
-    def _check_readable(self):
+    def _check_readable(self) -> None:
         """Check if the device is readable, raise OSError if not."""
         if not self.dev.isReadable():
             raise OSError("Trying to read unreadable file!")
 
-    def _check_writable(self):
+    def _check_writable(self) -> None:
         """Check if the device is writable, raise OSError if not."""
         if not self.writable():
             raise OSError("Trying to write to unwritable file!")
@@ -258,7 +270,7 @@ class PyQIODevice(io.BufferedIOBase):
             raise QtOSError(self.dev)
         return contextlib.closing(self)
 
-    def close(self):
+    def close(self) -> None:
         """Close the underlying device."""
         self.dev.close()
 
@@ -280,22 +292,22 @@ class PyQIODevice(io.BufferedIOBase):
         if not ok:
             raise QtOSError(self.dev, msg="seek failed!")
 
-    def truncate(self, size=None):  # pylint: disable=unused-argument
+    def truncate(self, size=None):
         raise io.UnsupportedOperation
 
     @property
-    def closed(self):
+    def closed(self) -> bool:
         return not self.dev.isOpen()
 
-    def flush(self):
+    def flush(self) -> None:
         self._check_open()
         self.dev.waitForBytesWritten(-1)
 
-    def isatty(self):
+    def isatty(self) -> bool:
         self._check_open()
         return False
 
-    def readable(self):
+    def readable(self) -> bool:
         return self.dev.isReadable()
 
     def readline(self, size=-1):
@@ -321,18 +333,18 @@ class PyQIODevice(io.BufferedIOBase):
             raise QtOSError(self.dev)
         return buf
 
-    def seekable(self):
+    def seekable(self) -> bool:
         return not self.dev.isSequential()
 
-    def tell(self):
+    def tell(self) -> int:
         self._check_open()
         self._check_random()
         return self.dev.pos()
 
-    def writable(self):
+    def writable(self) -> bool:
         return self.dev.isWritable()
 
-    def write(self, b):
+    def write(self, b: bytes) -> int:
         self._check_open()
         self._check_writable()
         num = self.dev.write(b)
@@ -356,7 +368,7 @@ class QtValueError(ValueError):
 
     """Exception which gets raised by ensure_valid."""
 
-    def __init__(self, obj):
+    def __init__(self, obj: QObject) -> None:
         try:
             self.reason = obj.errorString()
         except AttributeError:
@@ -374,7 +386,7 @@ class EventLoop(QEventLoop):
     Raises an exception when doing exec_() multiple times.
     """
 
-    def __init__(self, parent=None):
+    def __init__(self, parent: QObject = None) -> None:
         super().__init__(parent)
         self._executing = False
 
@@ -383,5 +395,6 @@ class EventLoop(QEventLoop):
         if self._executing:
             raise AssertionError("Eventloop is already running!")
         self._executing = True
-        super().exec_(flags)
+        status = super().exec_(flags)
         self._executing = False
+        return status
